from django import forms
from django.core import exceptions
from django.forms.models import inlineformset_factory
from django.utils.translation import gettext_lazy as _

from oscar.core.loading import get_classes, get_model

Product = get_model("catalogue", "Product")
ProductClass = get_model("catalogue", "ProductClass")
ProductAttribute = get_model("catalogue", "ProductAttribute")
StockRecord = get_model("partner", "StockRecord")
ProductCategory = get_model("catalogue", "ProductCategory")
ProductImage = get_model("catalogue", "ProductImage")
ProductRecommendation = get_model("catalogue", "ProductRecommendation")
AttributeOptionGroup = get_model("catalogue", "AttributeOptionGroup")
AttributeOption = get_model("catalogue", "AttributeOption")

(
    StockRecordForm,
    ProductCategoryForm,
    ProductImageForm,
    ProductRecommendationForm,
    ProductAttributesForm,
    AttributeOptionForm,
) = get_classes(
    "dashboard.catalogue.forms",
    (
        "StockRecordForm",
        "ProductCategoryForm",
        "ProductImageForm",
        "ProductRecommendationForm",
        "ProductAttributesForm",
        "AttributeOptionForm",
    ),
)


BaseStockRecordFormSet = inlineformset_factory(
    Product, StockRecord, form=StockRecordForm, extra=1
)


class StockRecordFormSet(BaseStockRecordFormSet):
    def __init__(self, product_class, user, *args, **kwargs):
        self.user = user
        self.require_user_stockrecord = not user.is_staff
        self.product_class = product_class

        if not user.is_staff and "instance" in kwargs and "queryset" not in kwargs:
            kwargs.update(
                {
                    "queryset": StockRecord.objects.filter(
                        product=kwargs["instance"], partner__in=user.partners.all()
                    )
                }
            )

        super().__init__(*args, **kwargs)
        self.set_initial_data()

    def set_initial_data(self):
        """
        If user has only one partner associated, set the first
        stock record's partner to it. Can't pre-select for staff users as
        they're allowed to save a product without a stock record.

        This is intentionally done after calling __init__ as passing initial
        data to __init__ creates a form for each list item. So depending on
        whether we can pre-select the partner or not, we'd end up with 1 or 2
        forms for an unbound form.
        """
        if self.require_user_stockrecord:
            try:
                user_partner = self.user.partners.get()
            except (exceptions.ObjectDoesNotExist, exceptions.MultipleObjectsReturned):
                pass
            else:
                partner_field = self.forms[0].fields.get("partner", None)
                if partner_field and partner_field.initial is None:
                    partner_field.initial = user_partner

    def _construct_form(self, i, **kwargs):
        kwargs["product_class"] = self.product_class
        kwargs["user"] = self.user
        return super()._construct_form(i, **kwargs)

    def clean(self):
        """
        If the user isn't a staff user, this validation ensures that at least
        one stock record's partner is associated with a users partners.
        """
        if any(self.errors):
            return
        if self.require_user_stockrecord:
            stockrecord_partners = set(
                [form.cleaned_data.get("partner", None) for form in self.forms]
            )
            user_partners = set(self.user.partners.all())
            if not user_partners & stockrecord_partners:
                raise exceptions.ValidationError(
                    _(
                        "At least one stock record must be set to a partner that"
                        " you're associated with."
                    )
                )


BaseProductCategoryFormSet = inlineformset_factory(
    Product, ProductCategory, form=ProductCategoryForm, extra=1, can_delete=True
)


class ProductCategoryFormSet(BaseProductCategoryFormSet):
    # pylint: disable=unused-argument
    def __init__(self, product_class, user, *args, **kwargs):
        # This function just exists to drop the extra arguments
        super().__init__(*args, **kwargs)

    def clean(self):
        if not self.instance.is_child and self.get_num_categories() == 0:
            raise forms.ValidationError(
                _("Stand-alone and parent products must have at least one category")
            )
        if self.instance.is_child and self.get_num_categories() > 0:
            raise forms.ValidationError(_("A child product should not have categories"))

    def get_num_categories(self):
        num_categories = 0
        for i in range(0, self.total_form_count()):
            form = self.forms[i]
            if (
                hasattr(form, "cleaned_data")
                and form.cleaned_data.get("category", None)
                and not form.cleaned_data.get("DELETE", False)
            ):
                num_categories += 1
        return num_categories


BaseProductImageFormSet = inlineformset_factory(
    Product, ProductImage, form=ProductImageForm, extra=2
)


class ProductImageFormSet(BaseProductImageFormSet):
    # pylint: disable=unused-argument
    def __init__(self, product_class, user, *args, **kwargs):
        super().__init__(*args, **kwargs)


BaseProductRecommendationFormSet = inlineformset_factory(
    Product,
    ProductRecommendation,
    form=ProductRecommendationForm,
    extra=5,
    fk_name="primary",
)


class ProductRecommendationFormSet(BaseProductRecommendationFormSet):
    # pylint: disable=unused-argument
    def __init__(self, product_class, user, *args, **kwargs):
        super().__init__(*args, **kwargs)


ProductAttributesFormSet = inlineformset_factory(
    ProductClass, ProductAttribute, form=ProductAttributesForm, extra=3
)


AttributeOptionFormSet = inlineformset_factory(
    AttributeOptionGroup, AttributeOption, form=AttributeOptionForm, extra=3
)



# Generated by Django 3.0.8 on 2020-08-01 07:17

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('order', '0010_auto_20200724_0909'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='orderdiscount',
            options={'ordering': ['pk'], 'verbose_name': 'Order Discount', 'verbose_name_plural': 'Order Discounts'},
        ),
        migrations.AlterModelOptions(
            name='ordernote',
            options={'ordering': ['-date_updated'], 'verbose_name': 'Order Note', 'verbose_name_plural': 'Order Notes'},
        ),
    ]



# -*- coding: utf-8 -*-
from decimal import Decimal as D

from django.core.validators import MinValueValidator
from django.db import models
from django.utils.translation import gettext_lazy as _

from oscar.core import loading, prices
from oscar.models.fields import AutoSlugField

Scale = loading.get_class("shipping.scales", "Scale")


class AbstractBase(models.Model):
    """
    Implements the interface declared by shipping.base.Base
    """

    code = AutoSlugField(
        _("Slug"), max_length=128, unique=True, populate_from="name", db_index=True
    )
    name = models.CharField(_("Name"), max_length=128, unique=True, db_index=True)
    description = models.TextField(_("Description"), blank=True)

    # We allow shipping methods to be linked to a specific set of countries
    countries = models.ManyToManyField(
        "address.Country", blank=True, verbose_name=_("Countries")
    )

    # We need this to mimic the interface of the Base shipping method
    is_discounted = False

    class Meta:
        abstract = True
        app_label = "shipping"
        ordering = ["name"]
        verbose_name = _("Shipping Method")
        verbose_name_plural = _("Shipping Methods")

    def __str__(self):
        return self.name

    # pylint: disable=unused-argument
    def discount(self, basket):
        """
        Return the discount on the standard shipping charge
        """
        # This method is identical to the Base.discount().
        return D("0.00")


class AbstractOrderAndItemCharges(AbstractBase):
    """
    Standard shipping method

    This method has two components:
    * a charge per order
    * a charge per item

    Many sites use shipping logic which fits into this system.  However, for
    more complex shipping logic, a custom shipping method object will need to
    be provided that subclasses ShippingMethod.
    """

    price_per_order = models.DecimalField(
        _("Price per order"), decimal_places=2, max_digits=12, default=D("0.00")
    )
    price_per_item = models.DecimalField(
        _("Price per item"), decimal_places=2, max_digits=12, default=D("0.00")
    )

    # If basket value is above this threshold, then shipping is free
    free_shipping_threshold = models.DecimalField(
        _("Free Shipping"), decimal_places=2, max_digits=12, blank=True, null=True
    )

    class Meta(AbstractBase.Meta):
        abstract = True
        app_label = "shipping"
        verbose_name = _("Order and Item Charge")
        verbose_name_plural = _("Order and Item Charges")

    def calculate(self, basket):
        if (
            self.free_shipping_threshold is not None
            and basket.total_incl_tax >= self.free_shipping_threshold
        ):
            return prices.Price(
                currency=basket.currency, excl_tax=D("0.00"), incl_tax=D("0.00")
            )

        charge = self.price_per_order
        for line in basket.all_lines():
            if line.product.is_shipping_required:
                charge += line.quantity * self.price_per_item

        # Zero tax is assumed...
        return prices.Price(currency=basket.currency, excl_tax=charge, incl_tax=charge)


class AbstractWeightBased(AbstractBase):
    # The attribute code to use to look up the weight of a product
    weight_attribute = "weight"

    # The default weight to use (in kg) when a product doesn't have a weight
    # attribute.
    default_weight = models.DecimalField(
        _("Default Weight"),
        decimal_places=3,
        max_digits=12,
        default=D("0.000"),
        validators=[MinValueValidator(D("0.00"))],
        help_text=_("Default product weight in kg when no weight attribute is defined"),
    )

    class Meta(AbstractBase.Meta):
        abstract = True
        app_label = "shipping"
        verbose_name = _("Weight-based Shipping Method")
        verbose_name_plural = _("Weight-based Shipping Methods")

    def calculate(self, basket):
        # Note, when weighing the basket, we don't check whether the item
        # requires shipping or not.  It is assumed that if something has a
        # weight, then it requires shipping.
        scale = Scale(
            attribute_code=self.weight_attribute, default_weight=self.default_weight
        )
        weight = scale.weigh_basket(basket)
        charge = self.get_charge(weight)

        # Zero tax is assumed...
        return prices.Price(currency=basket.currency, excl_tax=charge, incl_tax=charge)

    def get_charge(self, weight):
        """
        Calculates shipping charges for a given weight.

        If there is one or more matching weight band for a given weight, the
        charge of the closest matching weight band is returned.

        If the weight exceeds the top weight band, the top weight band charge
        is added until a matching weight band is found. This models the concept
        of "sending as many of the large boxes as needed".

        Please note that it is assumed that the closest matching weight band
        is the most cost-effective one, and that the top weight band is more
        cost effective than e.g. sending out two smaller parcels.
        Without that assumption, determining the cheapest shipping solution
        becomes an instance of the bin packing problem. The bin packing problem
        is NP-hard and solving it is left as an exercise to the reader.
        """
        weight = D(weight)  # weight really should be stored as a decimal
        if not self.bands.exists():
            return D("0.00")

        top_band = self.top_band
        if weight <= top_band.upper_limit:
            band = self.get_band_for_weight(weight)
            return band.charge
        else:
            quotient, remaining_weight = divmod(weight, top_band.upper_limit)
            if remaining_weight:
                remainder_band = self.get_band_for_weight(remaining_weight)
                return quotient * top_band.charge + remainder_band.charge
            else:
                return quotient * top_band.charge

    def get_band_for_weight(self, weight):
        """
        Return the closest matching weight band for a given weight.
        """
        return (
            self.bands.filter(upper_limit__gte=weight).order_by("upper_limit").first()
        )

    @property
    def num_bands(self):
        return self.bands.count()

    @property
    def top_band(self):
        return self.bands.order_by("-upper_limit").first()


class AbstractWeightBand(models.Model):
    """
    Represents a weight band which are used by the WeightBasedShipping method.
    """

    method = models.ForeignKey(
        "shipping.WeightBased",
        on_delete=models.CASCADE,
        related_name="bands",
        verbose_name=_("Method"),
    )
    upper_limit = models.DecimalField(
        _("Upper Limit"),
        decimal_places=3,
        max_digits=12,
        db_index=True,
        validators=[MinValueValidator(D("0.00"))],
        help_text=_(
            "Enter upper limit of this weight band in kg. The lower "
            "limit will be determined by the other weight bands."
        ),
    )
    charge = models.DecimalField(
        _("Charge"),
        decimal_places=2,
        max_digits=12,
        validators=[MinValueValidator(D("0.00"))],
    )

    @property
    def weight_from(self):
        lower_bands = self.method.bands.filter(
            upper_limit__lt=self.upper_limit
        ).order_by("-upper_limit")
        if not lower_bands:
            return D("0.000")
        return lower_bands[0].upper_limit

    @property
    def weight_to(self):
        return self.upper_limit

    class Meta:
        abstract = True
        app_label = "shipping"
        ordering = ["method", "upper_limit"]
        verbose_name = _("Weight Band")
        verbose_name_plural = _("Weight Bands")

    def __str__(self):
        return _("Charge for weights up to %s kg") % (self.upper_limit,)



from oscar.apps.address.abstract_models import AbstractPartnerAddress
from oscar.apps.partner.abstract_models import (
    AbstractPartner,
    AbstractStockAlert,
    AbstractStockRecord,
)
from oscar.core.loading import is_model_registered

__all__ = []


if not is_model_registered("partner", "Partner"):

    class Partner(AbstractPartner):
        pass

    __all__.append("Partner")


if not is_model_registered("partner", "PartnerAddress"):

    class PartnerAddress(AbstractPartnerAddress):
        pass

    __all__.append("PartnerAddress")


if not is_model_registered("partner", "StockRecord"):

    class StockRecord(AbstractStockRecord):
        pass

    __all__.append("StockRecord")


if not is_model_registered("partner", "StockAlert"):

    class StockAlert(AbstractStockAlert):
        pass

    __all__.append("StockAlert")



from django.db.models import F

from oscar.core.loading import get_model

ProductRecord = get_model("analytics", "ProductRecord")


class Calculator(object):
    # Map of field name to weight
    weights = {"num_views": 1, "num_basket_additions": 3, "num_purchases": 5}

    def __init__(self, logger):
        self.logger = logger

    def run(self):
        self.calculate_scores()

    def calculate_scores(self):
        self.logger.info("Calculating product scores")
        total_weight = float(sum(self.weights.values()))
        weighted_fields = [self.weights[name] * F(name) for name in self.weights.keys()]
        ProductRecord.objects.update(score=sum(weighted_fields) / total_weight)



from django import template

register = template.Library()


@register.simple_tag
def purchase_info_for_product(request, product):
    if product.is_parent:
        return request.strategy.fetch_for_parent(product)

    return request.strategy.fetch_for_product(product)


@register.simple_tag
def purchase_info_for_line(request, line):
    return request.strategy.fetch_for_line(line)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('basket', '0004_auto_20141007_2032'),
    ]

    operations = [
        migrations.AlterField(
            model_name='basket',
            name='vouchers',
            field=models.ManyToManyField(to='voucher.Voucher', verbose_name='Vouchers', blank=True),
        ),
    ]



from django.conf import settings


def checkout(request):
    anon_checkout_allowed = getattr(settings, "OSCAR_ALLOW_ANON_CHECKOUT", False)
    return {"anon_checkout_allowed": anon_checkout_allowed}



from django.conf import settings
from django.contrib import messages
from django.core.exceptions import ValidationError
from django.http import HttpResponseRedirect
from django.template.loader import render_to_string
from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from django.views import generic
from django.views.generic import ListView

from oscar.core.loading import get_classes, get_model
from oscar.core.utils import slugify
from oscar.core.validators import URLDoesNotExistValidator

FlatPage = get_model("flatpages", "FlatPage")
Site = get_model("sites", "Site")
PageSearchForm, PageUpdateForm = get_classes(
    "dashboard.pages.forms", ("PageSearchForm", "PageUpdateForm")
)


class PageListView(ListView):
    """
    View for listing all existing flatpages.
    """

    template_name = "oscar/dashboard/pages/index.html"
    model = FlatPage
    form_class = PageSearchForm
    paginate_by = settings.OSCAR_DASHBOARD_ITEMS_PER_PAGE
    desc_template = "%(main_filter)s %(title_filter)s"

    def get_queryset(self):
        """
        Get queryset of all flatpages to be displayed. If a
        search term is specified in the search form, it will be used
        to filter the queryset.
        """
        # pylint: disable=attribute-defined-outside-init
        self.desc_ctx = {
            "main_filter": _("All pages"),
            "title_filter": "",
        }
        queryset = self.model.objects.all().order_by("title")

        # pylint: disable=attribute-defined-outside-init
        self.form = self.form_class(self.request.GET)
        if not self.form.is_valid():
            return queryset

        data = self.form.cleaned_data

        if data["title"]:
            queryset = queryset.filter(title__icontains=data["title"])
            self.desc_ctx["title_filter"] = (
                _(" with title containing '%s'") % data["title"]
            )

        return queryset

    def get_context_data(self, **kwargs):
        """
        Get context data with *form* and *queryset_description* data
        added to it.
        """
        context = super().get_context_data(**kwargs)
        context["form"] = self.form
        context["queryset_description"] = self.desc_template % self.desc_ctx
        return context


class PageCreateUpdateMixin(object):
    template_name = "oscar/dashboard/pages/update.html"
    model = FlatPage
    form_class = PageUpdateForm
    context_object_name = "page"

    def get_success_url(self):
        msg = render_to_string(
            "oscar/dashboard/pages/messages/saved.html", {"page": self.object}
        )
        messages.success(self.request, msg, extra_tags="safe noicon")
        return reverse("dashboard:page-list")

    def form_valid(self, form):
        # Ensure saved page is added to the current site
        page = form.save()
        if not page.sites.exists():
            page.sites.add(Site.objects.get_current())
        self.object = page
        return HttpResponseRedirect(self.get_success_url())


class PageCreateView(PageCreateUpdateMixin, generic.CreateView):
    def get_context_data(self, **kwargs):
        ctx = super().get_context_data(**kwargs)
        ctx["title"] = _("Create New Page")
        return ctx

    def form_valid(self, form):
        """
        Store new flatpage from form data.
        Additionally, if URL is left blank, a slugified
        version of the title will be used as URL after checking
        if it is valid.
        """
        # if no URL is specified, generate from title
        page = form.save(commit=False)

        if not page.url:
            page.url = "/%s/" % slugify(page.title)

        try:
            URLDoesNotExistValidator()(page.url)
        except ValidationError:
            pass
        else:
            return super().form_valid(form)

        ctx = self.get_context_data()
        ctx["form"] = form
        return self.render_to_response(ctx)


class PageUpdateView(PageCreateUpdateMixin, generic.UpdateView):
    """
    View for updating flatpages from the dashboard.
    """

    def get_context_data(self, **kwargs):
        ctx = super().get_context_data(**kwargs)
        ctx["title"] = self.object.title
        return ctx


class PageDeleteView(generic.DeleteView):
    template_name = "oscar/dashboard/pages/delete.html"
    model = FlatPage

    def get_success_url(self):
        messages.success(self.request, _("Deleted page '%s'") % self.object.title)
        return reverse("dashboard:page-list")



# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-08-02 10:58
from __future__ import unicode_literals

from django.db import migrations, models
import oscar.apps.catalogue.reviews.utils


class Migration(migrations.Migration):

    dependencies = [
        ('reviews', '0002_update_email_length'),
    ]

    operations = [
        migrations.AlterField(
            model_name='productreview',
            name='status',
            field=models.SmallIntegerField(choices=[(0, 'Requires moderation'), (1, 'Approved'), (2, 'Rejected')], default=oscar.apps.catalogue.reviews.utils.get_default_review_status, verbose_name='Status'),
        ),
    ]



# Generated by Django 2.1 on 2019-06-17 17:42

from django.db import migrations, models


def migrate_product_options(apps, schema_editor):
    """
    Migrate product Option.type field to required
    Set Option.type='text'
    """
    Option = apps.get_model('catalogue', 'Option')
    for option in Option.objects.all():
        if option.type == "Required":
            option.required = True
        option.type = 'text'
        option.save()


class Migration(migrations.Migration):

    dependencies = [
        ('catalogue', '0018_auto_20191220_0920'),
    ]

    operations = [
        migrations.AddField(
            model_name='option',
            name='required',
            field=models.BooleanField(default=False, verbose_name='Is option required?'),
        ),
        migrations.RunPython(migrate_product_options, migrations.RunPython.noop),
        migrations.AlterField(
            model_name='option',
            name='type',
            field=models.CharField(choices=[('text', 'Text'), ('integer', 'Integer'), ('boolean', 'True / False'), ('float', 'Float'), ('date', 'Date')], default='text', max_length=255, verbose_name='Type'),
        ),
    ]



from django.db import models


class Band(models.Model):
    name = models.CharField(max_length=100)


class Member(models.Model):
    name = models.CharField(max_length=100)
    band = models.ForeignKey(Band, models.CASCADE)



from django.db import models


class OpenBasketManager(models.Manager):
    """For searching/creating OPEN baskets only."""

    status_filter = "Open"

    def get_queryset(self):
        return super().get_queryset().filter(status=self.status_filter)

    def get_or_create(self, **kwargs):
        return self.get_queryset().get_or_create(status=self.status_filter, **kwargs)


class SavedBasketManager(models.Manager):
    """For searching/creating SAVED baskets only."""

    status_filter = "Saved"

    def get_queryset(self):
        return super().get_queryset().filter(status=self.status_filter)

    def create(self, **kwargs):
        return self.get_queryset().create(status=self.status_filter, **kwargs)

    def get_or_create(self, **kwargs):
        return self.get_queryset().get_or_create(status=self.status_filter, **kwargs)



# -*- coding: utf-8 -*-

from django.core.management.base import BaseCommand

from oscar.core.loading import get_model

Product = get_model("catalogue", "Product")


class Command(BaseCommand):
    help = """Update the denormalised reviews average on all Product instances.
              Should only be necessary when changing to e.g. a weight-based
              rating."""

    def handle(self, *args, **options):
        # Iterate over all Products (not just ones with reviews)
        products = Product.objects.all()
        for product in products:
            product.update_rating()
        self.stdout.write("Successfully updated %s products\n" % products.count())



# Generated by Django 4.2.15 on 2024-09-11 10:37

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("catalogue", "0027_attributeoption_code_attributeoptiongroup_code_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="product",
            name="priority",
            field=models.SmallIntegerField(
                db_index=True,
                default=0,
                help_text="The highest priority products are shown first",
                validators=[django.core.validators.MinValueValidator(-1)],
                verbose_name="Priority",
            ),
        ),
    ]



from django.conf import settings
from django.core.validators import RegexValidator
from django.db import models
from django.template import engines
from django.template.exceptions import TemplateDoesNotExist
from django.template.loader import get_template
from django.utils.translation import gettext_lazy as _

from oscar.apps.communication.managers import CommunicationTypeManager
from oscar.core.compat import AUTH_USER_MODEL
from oscar.models.fields import AutoSlugField


class AbstractEmail(models.Model):
    """
    This is a record of an email sent to a customer.
    """

    user = models.ForeignKey(
        AUTH_USER_MODEL,
        on_delete=models.CASCADE,
        related_name="emails",
        verbose_name=_("User"),
        null=True,
    )
    email = models.EmailField(_("Email Address"), null=True, blank=True)
    subject = models.TextField(_("Subject"), max_length=255)
    body_text = models.TextField(_("Body Text"))
    body_html = models.TextField(_("Body HTML"), blank=True)
    date_sent = models.DateTimeField(_("Date Sent"), auto_now_add=True)

    class Meta:
        abstract = True
        app_label = "communication"
        ordering = ["-date_sent"]
        verbose_name = _("Email")
        verbose_name_plural = _("Emails")

    def __str__(self):
        if self.user:
            return _("Email to %(user)s with subject '%(subject)s'") % {
                "user": self.user.get_username(),
                "subject": self.subject,
            }
        else:
            return _("Email to %(email)s with subject '%(subject)s'") % {
                "email": self.email,
                "subject": self.subject,
            }


class AbstractCommunicationEventType(models.Model):
    """
    A 'type' of communication.  Like an order confirmation email.
    """

    #: Code used for looking up this event programmatically.
    # e.g. PASSWORD_RESET. AutoSlugField uppercases the code for us because
    # it's a useful convention that's been enforced in previous Oscar versions
    code = AutoSlugField(
        _("Code"),
        max_length=128,
        unique=True,
        populate_from="name",
        separator="_",
        uppercase=True,
        editable=True,
        validators=[
            RegexValidator(
                regex=r"^[A-Z_][0-9A-Z_]*$",
                message=_(
                    "Code can only contain the uppercase letters (A-Z), "
                    "digits, and underscores, and can't start with a digit."
                ),
            )
        ],
        help_text=_("Code used for looking up this event programmatically"),
    )

    #: Name is the friendly description of an event for use in the admin
    name = models.CharField(_("Name"), max_length=255, db_index=True)

    # We allow communication types to be categorised
    # For backwards-compatibility, the choice values are quite verbose
    ORDER_RELATED = "Order related"
    USER_RELATED = "User related"
    CATEGORY_CHOICES = (
        (ORDER_RELATED, _("Order related")),
        (USER_RELATED, _("User related")),
    )

    category = models.CharField(
        _("Category"), max_length=255, default=ORDER_RELATED, choices=CATEGORY_CHOICES
    )

    # Template content for emails
    # NOTE: There's an intentional distinction between None and ''. None
    # instructs Oscar to look for a file-based template, '' is just an empty
    # template.
    email_subject_template = models.CharField(
        _("Email Subject Template"), max_length=255, blank=True, null=True
    )
    email_body_template = models.TextField(
        _("Email Body Template"), blank=True, null=True
    )
    email_body_html_template = models.TextField(
        _("Email Body HTML Template"),
        blank=True,
        null=True,
        help_text=_("HTML template"),
    )

    # Template content for SMS messages
    sms_template = models.CharField(
        _("SMS Template"),
        max_length=170,
        blank=True,
        null=True,
        help_text=_("SMS template"),
    )

    date_created = models.DateTimeField(_("Date Created"), auto_now_add=True)
    date_updated = models.DateTimeField(_("Date Updated"), auto_now=True)

    objects = CommunicationTypeManager()

    # File templates
    email_subject_template_file = "oscar/communication/emails/commtype_%s_subject.txt"
    email_body_template_file = "oscar/communication/emails/commtype_%s_body.txt"
    email_body_html_template_file = "oscar/communication/emails/commtype_%s_body.html"
    sms_template_file = "oscar/communication/sms/commtype_%s_body.txt"

    class Meta:
        abstract = True
        app_label = "communication"
        ordering = ["name"]
        verbose_name = _("Communication event type")
        verbose_name_plural = _("Communication event types")

    def get_messages(self, ctx=None):
        """
        Return a dict of templates with the context merged in

        We look first at the field templates but fail over to
        a set of file templates that follow a conventional path.
        """
        code = self.code.lower()

        # Build a dict of message name to Template instances
        templates = {
            "subject": "email_subject_template",
            "body": "email_body_template",
            "html": "email_body_html_template",
            "sms": "sms_template",
        }
        for name, attr_name in templates.items():
            field = getattr(self, attr_name, None)
            if field is not None:
                # Template content is in a model field
                templates[name] = engines["django"].from_string(field)
            else:
                # Model field is empty - look for a file template
                template_name = getattr(self, "%s_file" % attr_name) % code
                try:
                    templates[name] = get_template(template_name)
                except TemplateDoesNotExist:
                    templates[name] = None

        # Pass base URL for serving images within HTML emails
        if ctx is None:
            ctx = {}
        ctx["static_base_url"] = getattr(settings, "OSCAR_STATIC_BASE_URL", None)

        messages = {}
        for name, template in templates.items():
            # pylint: disable=no-member
            messages[name] = template.render(ctx) if template else ""

        # Ensure the email subject doesn't contain any newlines
        messages["subject"] = messages["subject"].replace("\n", "")
        messages["subject"] = messages["subject"].replace("\r", "")

        return messages

    def __str__(self):
        return self.name

    def is_order_related(self):
        return self.category == self.ORDER_RELATED

    def is_user_related(self):
        return self.category == self.USER_RELATED


class AbstractNotification(models.Model):
    recipient = models.ForeignKey(
        AUTH_USER_MODEL, on_delete=models.CASCADE, related_name="notifications"
    )

    # Not all notifications will have a sender.
    sender = models.ForeignKey(AUTH_USER_MODEL, on_delete=models.CASCADE, null=True)

    # HTML is allowed in this field as it can contain links
    subject = models.CharField(max_length=255)
    body = models.TextField()

    INBOX, ARCHIVE = "Inbox", "Archive"
    choices = ((INBOX, _("Inbox")), (ARCHIVE, _("Archive")))
    location = models.CharField(max_length=32, choices=choices, default=INBOX)

    date_sent = models.DateTimeField(auto_now_add=True)
    date_read = models.DateTimeField(blank=True, null=True)

    class Meta:
        abstract = True
        app_label = "communication"
        ordering = ("-date_sent",)
        verbose_name = _("Notification")
        verbose_name_plural = _("Notifications")

    def __str__(self):
        return self.subject

    def archive(self):
        self.location = self.ARCHIVE
        self.save()

    archive.alters_data = True

    @property
    def is_read(self):
        return self.date_read is not None



from django.utils.translation import gettext_lazy as _

from oscar.core.loading import get_class

ReportCSVFormatter = get_class("dashboard.reports.reports", "ReportCSVFormatter")


class OrderDiscountCSVFormatter(ReportCSVFormatter):
    filename_template = "order-discounts-for-offer-%s.csv"

    def generate_csv(self, response, order_discounts):
        writer = self.get_csv_writer(response)
        header_row = [_("Order number"), _("Order date"), _("Order total"), _("Cost")]
        writer.writerow(header_row)
        for order_discount in order_discounts:
            order = order_discount.order
            row = [
                order.number,
                self.format_datetime(order.date_placed),
                order.total_incl_tax,
                order_discount.amount,
            ]
            writer.writerow(row)

    def filename(self, offer):
        return self.filename_template % offer.id



# pylint: disable=W0621, unused-argument

from decimal import Decimal as D

from django.conf import settings
from django.utils.translation import gettext_lazy as _

from oscar.core.loading import get_class, get_classes, get_model
from oscar.templatetags.currency_filters import currency

Benefit = get_model("offer", "Benefit")
BasketDiscount, SHIPPING_DISCOUNT, ZERO_DISCOUNT = get_classes(
    "offer.results", ["BasketDiscount", "SHIPPING_DISCOUNT", "ZERO_DISCOUNT"]
)
CoverageCondition, ValueCondition = get_classes(
    "offer.conditions", ["CoverageCondition", "ValueCondition"]
)
range_anchor = get_class("offer.utils", "range_anchor")

__all__ = [
    "PercentageDiscountBenefit",
    "AbsoluteDiscountBenefit",
    "FixedUnitDiscountBenefit",
    "FixedPriceBenefit",
    "ShippingBenefit",
    "MultibuyDiscountBenefit",
    "ShippingAbsoluteDiscountBenefit",
    "ShippingFixedPriceBenefit",
    "ShippingPercentageDiscountBenefit",
]


def apply_discount(line, discount, quantity, offer=None, incl_tax=None):
    """
    Apply a given discount to the passed basket
    """
    # use OSCAR_OFFERS_INCL_TAX setting if incl_tax is left unspecified.
    incl_tax = incl_tax if incl_tax is not None else settings.OSCAR_OFFERS_INCL_TAX
    line.discount(discount, quantity, incl_tax=incl_tax, offer=offer)


class PercentageDiscountBenefit(Benefit):
    """
    An offer benefit that gives a percentage discount
    """

    _description = _("%(value)s%% discount on %(range)s")

    @property
    def name(self):
        return self._description % {"value": self.value, "range": self.range.name}

    @property
    def description(self):
        return self._description % {
            "value": self.value,
            "range": range_anchor(self.range),
        }

    class Meta:
        app_label = "offer"
        proxy = True
        verbose_name = _("Percentage discount benefit")
        verbose_name_plural = _("Percentage discount benefits")

    # pylint: disable=unused-argument
    def apply(
        self,
        basket,
        condition,
        offer,
        discount_percent=None,
        max_total_discount=None,
        **kwargs,
    ):
        if discount_percent is None:
            discount_percent = self.value

        discount_amount_available = max_total_discount

        line_tuples = self.get_applicable_lines(offer, basket)
        discount_percent = min(discount_percent, D("100.0"))
        discount = D("0.00")
        affected_items = 0
        max_affected_items = self._effective_max_affected_items()
        for price, line in line_tuples:
            affected_items += line.quantity_with_offer_discount(offer)
            if affected_items >= max_affected_items:
                break
            if discount_amount_available == 0:
                break

            quantity_affected = min(
                line.quantity_without_offer_discount(offer),
                max_affected_items - affected_items,
            )
            if quantity_affected <= 0:
                break

            line_discount = (
                discount_percent / D("100.0") * price * int(quantity_affected)
            )

            if discount_amount_available is not None:
                line_discount = min(line_discount, discount_amount_available)
                discount_amount_available -= line_discount

            apply_discount(line, line_discount, quantity_affected, offer)

            affected_items += quantity_affected
            discount += line_discount

        return BasketDiscount(self.round(discount, basket.currency))


class AbsoluteDiscountBenefit(Benefit):
    """
    An offer benefit that gives an absolute discount
    """

    _description = _("%(value)s discount on %(range)s")

    @property
    def name(self):
        return self._description % {
            "value": currency(self.value),
            "range": self.range.name.lower(),
        }

    @property
    def description(self):
        return self._description % {
            "value": currency(self.value),
            "range": range_anchor(self.range),
        }

    class Meta:
        app_label = "offer"
        proxy = True
        verbose_name = _("Absolute discount benefit")
        verbose_name_plural = _("Absolute discount benefits")

    def apply(
        self,
        basket,
        condition,
        offer,
        discount_amount=None,
        max_total_discount=None,
        **kwargs,
    ):
        if discount_amount is None:
            discount_amount = self.value

        # Fetch basket lines that are in the range and available to be used in
        # an offer.
        line_tuples = self.get_applicable_lines(offer, basket)

        # Determine which lines can have the discount applied to them
        max_affected_items = self._effective_max_affected_items()
        num_affected_items = 0
        affected_items_total = D("0.00")
        lines_to_discount = []
        for price, line in line_tuples:
            if num_affected_items >= max_affected_items:
                break
            qty = min(
                line.quantity_without_offer_discount(offer),
                max_affected_items - num_affected_items,
            )
            lines_to_discount.append((line, price, qty))
            num_affected_items += qty
            affected_items_total += qty * price

        # Ensure we don't try to apply a discount larger than the total of the
        # matching items.
        discount = min(discount_amount, affected_items_total)
        if max_total_discount is not None:
            discount = min(discount, max_total_discount)

        if discount == 0:
            return ZERO_DISCOUNT

        # spreading the discount is a policy decision that may not apply

        # Apply discount equally amongst them
        applied_discount = D("0.00")
        last_line_idx = len(lines_to_discount) - 1
        for i, (line, price, qty) in enumerate(lines_to_discount):
            if i == last_line_idx:
                # If last line, then take the delta as the discount to ensure
                # the total discount is correct and doesn't mismatch due to
                # rounding.
                line_discount = discount - applied_discount
            else:
                # Calculate a weighted discount for the line
                line_discount = self.round(
                    ((price * qty) / affected_items_total) * discount, basket.currency
                )
            apply_discount(line, line_discount, qty, offer)
            applied_discount += line_discount

        return BasketDiscount(discount)


class FixedUnitDiscountBenefit(AbsoluteDiscountBenefit):
    """
    An offer benefit that gives an absolute discount on each applicable product.
    """

    class Meta:
        app_label = "offer"
        proxy = True
        verbose_name = _("Fixed unit discount benefit")
        verbose_name_plural = _("Fixed unit discount benefits")

    def get_lines_to_discount(self, offer, line_tuples):
        # Determine which lines can have the discount applied to them
        max_affected_items = self._effective_max_affected_items()
        num_affected_items = 0
        lines_to_discount = []
        for price, line in line_tuples:
            if num_affected_items >= max_affected_items:
                break
            qty = min(
                line.quantity_without_offer_discount(offer),
                max_affected_items - num_affected_items,
            )
            lines_to_discount.append((line, price, qty))
            num_affected_items += qty
        return lines_to_discount

    def apply(
        self,
        basket,
        condition,
        offer,
        discount_amount=None,
        max_total_discount=None,
        **kwargs,
    ):
        # Fetch basket lines that are in the range and available to be used in an offer.
        line_tuples = self.get_applicable_lines(offer, basket)
        lines_to_discount = self.get_lines_to_discount(offer, line_tuples)

        applied_discount = D("0.00")
        for line, price, qty in lines_to_discount:
            # If price is less than the fixed discount, then it will be free.
            line_discount = min(price * qty, self.value * qty)
            apply_discount(line, line_discount, qty, offer)
            applied_discount += line_discount

        return BasketDiscount(applied_discount)


class FixedPriceBenefit(Benefit):
    """
    An offer benefit that gives the items in the condition for a
    fixed price.  This is useful for "bundle" offers.

    Note that we ignore the benefit range here and only give a fixed price
    for the products in the condition range.  The condition cannot be a value
    condition.

    We also ignore the max_affected_items setting.
    """

    _description = _("The products that meet the condition are sold for %(amount)s")

    @property
    def name(self):
        return self._description % {"amount": currency(self.value)}

    class Meta:
        app_label = "offer"
        proxy = True
        verbose_name = _("Fixed price benefit")
        verbose_name_plural = _("Fixed price benefits")

    def apply(self, basket, condition, offer, **kwargs):
        if isinstance(condition, ValueCondition):
            return ZERO_DISCOUNT

        # Fetch basket lines that are in the range and available to be used in
        # an offer.
        line_tuples = self.get_applicable_lines(offer, basket, range=condition.range)
        if not line_tuples:
            return ZERO_DISCOUNT

        # Determine the lines to consume
        num_permitted = int(condition.value)
        num_affected = 0
        value_affected = D("0.00")
        covered_lines = []
        for price, line in line_tuples:
            if isinstance(condition, CoverageCondition):
                quantity_affected = 1
            else:
                quantity_affected = min(
                    line.quantity_without_offer_discount(offer),
                    num_permitted - num_affected,
                )
            num_affected += quantity_affected
            value_affected += quantity_affected * price
            covered_lines.append((price, line, quantity_affected))
            if num_affected >= num_permitted:
                break
        discount = max(value_affected - self.value, D("0.00"))
        if not discount:
            return ZERO_DISCOUNT

        # Apply discount to the affected lines
        discount_applied = D("0.00")
        last_line = covered_lines[-1][1]
        for price, line, quantity in covered_lines:
            if line == last_line:
                # If last line, we just take the difference to ensure that
                # rounding doesn't lead to an off-by-one error
                line_discount = discount - discount_applied
            else:
                line_discount = self.round(
                    discount * (price * quantity) / value_affected, basket.currency
                )
            apply_discount(line, line_discount, quantity, offer)
            discount_applied += line_discount
        return BasketDiscount(discount)


class MultibuyDiscountBenefit(Benefit):
    _description = _("Cheapest product from %(range)s is free")

    @property
    def name(self):
        return self._description % {"range": self.range.name.lower()}

    @property
    def description(self):
        return self._description % {"range": range_anchor(self.range)}

    class Meta:
        app_label = "offer"
        proxy = True
        verbose_name = _("Multibuy discount benefit")
        verbose_name_plural = _("Multibuy discount benefits")

    def apply(self, basket, condition, offer, **kwargs):
        line_tuples = self.get_applicable_lines(offer, basket)
        if not line_tuples:
            return ZERO_DISCOUNT

        # Cheapest line gives free product
        discount, line = line_tuples[0]
        if line.quantity_with_offer_discount(offer) == 0:
            apply_discount(line, discount, 1, offer)

            affected_lines = [(line, discount, 1)]
            condition.consume_items(offer, basket, affected_lines)

            return BasketDiscount(discount)
        else:
            return ZERO_DISCOUNT


# =================
# Shipping benefits
# =================


class ShippingBenefit(Benefit):
    def apply(self, basket, condition, offer, **kwargs):
        condition.consume_items(offer, basket, affected_lines=())
        return SHIPPING_DISCOUNT

    class Meta:
        app_label = "offer"
        proxy = True


class ShippingAbsoluteDiscountBenefit(ShippingBenefit):
    _description = _("%(amount)s off shipping cost")

    @property
    def name(self):
        return self._description % {"amount": currency(self.value)}

    class Meta:
        app_label = "offer"
        proxy = True
        verbose_name = _("Shipping absolute discount benefit")
        verbose_name_plural = _("Shipping absolute discount benefits")

    def shipping_discount(self, charge, currency=None):
        return min(charge, self.value)


class ShippingFixedPriceBenefit(ShippingBenefit):
    _description = _("Get shipping for %(amount)s")

    @property
    def name(self):
        return self._description % {"amount": currency(self.value)}

    class Meta:
        app_label = "offer"
        proxy = True
        verbose_name = _("Fixed price shipping benefit")
        verbose_name_plural = _("Fixed price shipping benefits")

    def shipping_discount(self, charge, currency=None):
        if charge < self.value:
            return D("0.00")
        return charge - self.value


class ShippingPercentageDiscountBenefit(ShippingBenefit):
    _description = _("%(value)s%% off of shipping cost")

    @property
    def name(self):
        return self._description % {"value": self.value}

    class Meta:
        app_label = "offer"
        proxy = True
        verbose_name = _("Shipping percentage discount benefit")
        verbose_name_plural = _("Shipping percentage discount benefits")

    def shipping_discount(self, charge, currency=None):
        discount = charge * self.value / D("100.0")
        return discount.quantize(D("0.01"))



# Generated by Django 3.1.4 on 2021-08-24 13:14

from django.db import migrations
import oscar.models.fields.slugfield


class Migration(migrations.Migration):

    dependencies = [
        ('catalogue', '0022_auto_20210210_0539'),
    ]

    operations = [
        migrations.AlterField(
            model_name='product',
            name='slug',
            field=oscar.models.fields.slugfield.SlugField(max_length=255, verbose_name='Slug'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2018-02-28 09:40
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
from django.utils.module_loading import import_string
from django.conf import settings

models_AutoField = import_string(settings.DEFAULT_AUTO_FIELD)


class Migration(migrations.Migration):

    dependencies = [
        ('offer', '0007_conditionaloffer_exclusive'),
        ('voucher', '0003_auto_20171212_0411'),
    ]

    operations = [
        migrations.CreateModel(
            name='VoucherSet',
            fields=[
                ('id', models_AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100, verbose_name='Name')),
                ('count', models.IntegerField(verbose_name='Number of vouchers')),
                ('code_length', models.IntegerField(default=12, verbose_name='Length of Code')),
                ('description', models.TextField(verbose_name='Description')),
                ('date_created', models.DateTimeField(auto_now_add=True)),
                ('start_datetime', models.DateTimeField(verbose_name='Start datetime')),
                ('end_datetime', models.DateTimeField(verbose_name='End datetime')),
                ('offer', models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='voucher_set', to='offer.ConditionalOffer', verbose_name='Offer')),
            ],
            options={
                'verbose_name': 'VoucherSet',
                'verbose_name_plural': 'VoucherSets',
                'get_latest_by': 'date_created',
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='voucher',
            name='voucher_set',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='vouchers', to='voucher.VoucherSet'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.8 on 2018-01-30 16:08
from __future__ import unicode_literals

from django.db import migrations

from oscar.core.loading import get_model


ORMUserAddress = get_model('address', 'UserAddress')


def regenerate_user_address_hashes(apps, schema_editor):
    MigrationUserAddress = apps.get_model('address', 'UserAddress')
    for user_address in MigrationUserAddress.objects.all():
        orm_user_address = ORMUserAddress.objects.get(id=user_address.id)
        user_address.hash = orm_user_address.generate_hash()
        user_address.save()


class Migration(migrations.Migration):

    dependencies = [
        ('address', '0004_auto_20170226_1122'),
    ]

    operations = [
        migrations.RunPython(regenerate_user_address_hashes, migrations.RunPython.noop)
    ]



from django.contrib.auth.decorators import login_required
from django.urls import path, re_path
from django.utils.translation import gettext_lazy as _
from django.views import generic

from oscar.core.application import OscarConfig
from oscar.core.loading import get_class


class CustomerConfig(OscarConfig):
    label = "customer"
    name = "oscar.apps.customer"
    verbose_name = _("Customer")

    namespace = "customer"

    # pylint: disable=attribute-defined-outside-init, reimported, unused-import
    def ready(self):
        from . import receivers
        from .alerts import receivers

        self.summary_view = get_class("customer.views", "AccountSummaryView")
        self.order_history_view = get_class("customer.views", "OrderHistoryView")
        self.order_detail_view = get_class("customer.views", "OrderDetailView")
        self.anon_order_detail_view = get_class(
            "customer.views", "AnonymousOrderDetailView"
        )
        self.anon_order_form_view = get_class(
            "customer.views", "AnonymousOrderFormView"
        )
        self.order_line_view = get_class("customer.views", "OrderLineView")

        self.address_list_view = get_class("customer.views", "AddressListView")
        self.address_create_view = get_class("customer.views", "AddressCreateView")
        self.address_update_view = get_class("customer.views", "AddressUpdateView")
        self.address_delete_view = get_class("customer.views", "AddressDeleteView")
        self.address_change_status_view = get_class(
            "customer.views", "AddressChangeStatusView"
        )

        self.email_list_view = get_class("customer.views", "EmailHistoryView")
        self.email_detail_view = get_class("customer.views", "EmailDetailView")
        self.login_view = get_class("customer.views", "AccountAuthView")
        self.logout_view = get_class("customer.views", "LogoutView")
        self.register_view = get_class("customer.views", "AccountRegistrationView")
        self.profile_view = get_class("customer.views", "ProfileView")
        self.profile_update_view = get_class("customer.views", "ProfileUpdateView")
        self.profile_delete_view = get_class("customer.views", "ProfileDeleteView")
        self.change_password_view = get_class("customer.views", "ChangePasswordView")

        self.notification_inbox_view = get_class(
            "communication.notifications.views", "InboxView"
        )
        self.notification_archive_view = get_class(
            "communication.notifications.views", "ArchiveView"
        )
        self.notification_update_view = get_class(
            "communication.notifications.views", "UpdateView"
        )
        self.notification_detail_view = get_class(
            "communication.notifications.views", "DetailView"
        )

        self.alert_list_view = get_class(
            "customer.alerts.views", "ProductAlertListView"
        )
        self.alert_create_view = get_class(
            "customer.alerts.views", "ProductAlertCreateView"
        )
        self.alert_confirm_view = get_class(
            "customer.alerts.views", "ProductAlertConfirmView"
        )
        self.alert_cancel_view = get_class(
            "customer.alerts.views", "ProductAlertCancelView"
        )

        self.wishlists_add_product_view = get_class(
            "customer.wishlists.views", "WishListAddProduct"
        )
        self.wishlists_list_view = get_class(
            "customer.wishlists.views", "WishListListView"
        )
        self.wishlists_detail_view = get_class(
            "customer.wishlists.views", "WishListDetailView"
        )
        self.wishlists_create_view = get_class(
            "customer.wishlists.views", "WishListCreateView"
        )
        self.wishlists_create_with_product_view = get_class(
            "customer.wishlists.views", "WishListCreateView"
        )
        self.wishlists_update_view = get_class(
            "customer.wishlists.views", "WishListUpdateView"
        )
        self.wishlists_delete_view = get_class(
            "customer.wishlists.views", "WishListDeleteView"
        )
        self.wishlists_remove_product_view = get_class(
            "customer.wishlists.views", "WishListRemoveProduct"
        )
        self.wishlists_move_product_to_another_view = get_class(
            "customer.wishlists.views", "WishListMoveProductToAnotherWishList"
        )

    def get_urls(self):
        urls = [
            # Login, logout and register doesn't require login
            path("login/", self.login_view.as_view(), name="login"),
            path("logout/", self.logout_view.as_view(), name="logout"),
            path("register/", self.register_view.as_view(), name="register"),
            path("", login_required(self.summary_view.as_view()), name="summary"),
            path(
                "change-password/",
                login_required(self.change_password_view.as_view()),
                name="change-password",
            ),
            # Profile
            path(
                "profile/",
                login_required(self.profile_view.as_view()),
                name="profile-view",
            ),
            path(
                "profile/edit/",
                login_required(self.profile_update_view.as_view()),
                name="profile-update",
            ),
            path(
                "profile/delete/",
                login_required(self.profile_delete_view.as_view()),
                name="profile-delete",
            ),
            # Order history
            path(
                "orders/",
                login_required(self.order_history_view.as_view()),
                name="order-list",
            ),
            re_path(
                r"^order-status-form/(?P<order_number>[\w-]*)/(?P<hash>[A-z0-9-_=:]+)/$",
                self.anon_order_form_view.as_view(),
                name="anon-order-form",
            ),
            re_path(
                r"^order-status/(?P<order_number>[\w-]*)/(?P<hash>[A-z0-9-_=:]+)/$",
                self.anon_order_detail_view.as_view(),
                name="anon-order",
            ),
            path(
                "orders/<str:order_number>/",
                login_required(self.order_detail_view.as_view()),
                name="order",
            ),
            path(
                "orders/<str:order_number>/<int:line_id>/",
                login_required(self.order_line_view.as_view()),
                name="order-line",
            ),
            # Address book
            path(
                "addresses/",
                login_required(self.address_list_view.as_view()),
                name="address-list",
            ),
            path(
                "addresses/add/",
                login_required(self.address_create_view.as_view()),
                name="address-create",
            ),
            path(
                "addresses/<int:pk>/",
                login_required(self.address_update_view.as_view()),
                name="address-detail",
            ),
            path(
                "addresses/<int:pk>/delete/",
                login_required(self.address_delete_view.as_view()),
                name="address-delete",
            ),
            re_path(
                r"^addresses/(?P<pk>\d+)/(?P<action>default_for_(billing|shipping))/$",
                login_required(self.address_change_status_view.as_view()),
                name="address-change-status",
            ),
            # Email history
            path(
                "emails/",
                login_required(self.email_list_view.as_view()),
                name="email-list",
            ),
            path(
                "emails/<int:email_id>/",
                login_required(self.email_detail_view.as_view()),
                name="email-detail",
            ),
            # Notifications
            # Redirect to notification inbox
            path(
                "notifications/",
                generic.RedirectView.as_view(
                    url="/accounts/notifications/inbox/", permanent=False
                ),
            ),
            path(
                "notifications/inbox/",
                login_required(self.notification_inbox_view.as_view()),
                name="notifications-inbox",
            ),
            path(
                "notifications/archive/",
                login_required(self.notification_archive_view.as_view()),
                name="notifications-archive",
            ),
            path(
                "notifications/update/",
                login_required(self.notification_update_view.as_view()),
                name="notifications-update",
            ),
            path(
                "notifications/<int:pk>/",
                login_required(self.notification_detail_view.as_view()),
                name="notifications-detail",
            ),
            # Alerts
            # Alerts can be setup by anonymous users: some views do not
            # require login
            path(
                "alerts/",
                login_required(self.alert_list_view.as_view()),
                name="alerts-list",
            ),
            path(
                "alerts/create/<int:pk>/",
                self.alert_create_view.as_view(),
                name="alert-create",
            ),
            path(
                "alerts/confirm/<str:key>/",
                self.alert_confirm_view.as_view(),
                name="alerts-confirm",
            ),
            path(
                "alerts/cancel/key/<str:key>/",
                self.alert_cancel_view.as_view(),
                name="alerts-cancel-by-key",
            ),
            path(
                "alerts/cancel/<int:pk>/",
                login_required(self.alert_cancel_view.as_view()),
                name="alerts-cancel-by-pk",
            ),
            # Wishlists
            path(
                "wishlists/",
                login_required(self.wishlists_list_view.as_view()),
                name="wishlists-list",
            ),
            path(
                "wishlists/add/<int:product_pk>/",
                login_required(self.wishlists_add_product_view.as_view()),
                name="wishlists-add-product",
            ),
            path(
                "wishlists/<str:key>/add/<int:product_pk>/",
                login_required(self.wishlists_add_product_view.as_view()),
                name="wishlists-add-product",
            ),
            path(
                "wishlists/create/",
                login_required(self.wishlists_create_view.as_view()),
                name="wishlists-create",
            ),
            path(
                "wishlists/create/with-product/<int:product_pk>/",
                login_required(self.wishlists_create_view.as_view()),
                name="wishlists-create-with-product",
            ),
            # Wishlists can be publicly shared, no login required
            path(
                "wishlists/<str:key>/",
                self.wishlists_detail_view.as_view(),
                name="wishlists-detail",
            ),
            path(
                "wishlists/<str:key>/update/",
                login_required(self.wishlists_update_view.as_view()),
                name="wishlists-update",
            ),
            path(
                "wishlists/<str:key>/delete/",
                login_required(self.wishlists_delete_view.as_view()),
                name="wishlists-delete",
            ),
            path(
                "wishlists/<str:key>/lines/<int:line_pk>/delete/",
                login_required(self.wishlists_remove_product_view.as_view()),
                name="wishlists-remove-product",
            ),
            path(
                "wishlists/<str:key>/products/<int:product_pk>/delete/",
                login_required(self.wishlists_remove_product_view.as_view()),
                name="wishlists-remove-product",
            ),
            path(
                "wishlists/<str:key>/lines/<int:line_pk>/move-to/<str:to_key>/",
                login_required(self.wishlists_move_product_to_another_view.as_view()),
                name="wishlists-move-product-to-another",
            ),
        ]

        return self.post_process_urls(urls)



import logging

from django.core.management.base import BaseCommand, CommandError

from oscar.core import customisation


class Command(BaseCommand):
    help = "Create a local version of one of Oscar's app so it can be customised"

    def add_arguments(self, parser):
        parser.add_argument("app_label", help="The application to fork")
        parser.add_argument("target_path", help="The path to copy the files to")
        parser.add_argument(
            "new_app_subpackage",
            nargs="?",
            help="Dotted path to the new app, inside the target path",
        )

    def handle(self, *args, **options):
        # Use a stdout logger
        logger = logging.getLogger(__name__)
        stream = logging.StreamHandler(self.stdout)
        logger.addHandler(stream)
        logger.setLevel(logging.DEBUG)

        app_label = options["app_label"]
        target_path = options["target_path"]
        new_app_subpackage = options["new_app_subpackage"]
        try:
            customisation.fork_app(app_label, target_path, new_app_subpackage, logger)
        except Exception as e:
            raise CommandError(str(e))
